import { __rest } from "tslib";
import { Injectable } from '@angular/core';
import { interpret, State, } from 'xstate';
import { filter, shareReplay, takeUntil } from 'rxjs/operators';
import { from, ReplaySubject } from 'rxjs';
export class XstateAngular {
    constructor() {
        this.unsubscribeSubject$ = new ReplaySubject(1);
        this.unsubscribe$ = this.unsubscribeSubject$.asObservable();
    }
    useMachine(machine, options = {}) {
        const { context, guards, actions, activities, services, delays, state } = options, interpreterOptions = __rest(options, ["context", "guards", "actions", "activities", "services", "delays", "state"]);
        const machineConfig = {
            context,
            guards,
            actions,
            activities,
            services,
            delays,
        };
        const createdMachine = machine.withConfig(machineConfig, Object.assign(Object.assign({}, machine.context), context));
        const service = interpret(createdMachine, interpreterOptions).start(state ? State.create(state) : undefined);
        const state$ = from(service).pipe(filter(({ changed }) => changed), shareReplay(1), takeUntil(this.unsubscribe$));
        return { state$, send: service.send, service };
    }
    ngOnDestroy() {
        this.service.stop();
        this.unsubscribeSubject$.next();
    }
}
XstateAngular.decorators = [
    { type: Injectable }
];
//# sourceMappingURL=useMachine.js.map