import { __rest } from 'tslib';
import { Injectable } from '@angular/core';
import { interpret, State } from 'xstate';
import { filter, shareReplay, takeUntil, finalize } from 'rxjs/operators';
import { ReplaySubject, from, BehaviorSubject } from 'rxjs';

class XstateAngular {
    constructor() {
        this.unsubscribeSubject$ = new ReplaySubject(1);
        this.unsubscribe$ = this.unsubscribeSubject$.asObservable();
    }
    useMachine(machine, options = {}) {
        const { context, guards, actions, activities, services, delays, state } = options, interpreterOptions = __rest(options, ["context", "guards", "actions", "activities", "services", "delays", "state"]);
        const machineConfig = {
            context,
            guards,
            actions,
            activities,
            services,
            delays,
        };
        const createdMachine = machine.withConfig(machineConfig, Object.assign(Object.assign({}, machine.context), context));
        const service = interpret(createdMachine, interpreterOptions).start(state ? State.create(state) : undefined);
        const state$ = from(service).pipe(filter(({ changed }) => changed), shareReplay(1), takeUntil(this.unsubscribe$));
        return { state$, send: service.send, service };
    }
    ngOnDestroy() {
        this.service.stop();
        this.unsubscribeSubject$.next();
    }
}
XstateAngular.decorators = [
    { type: Injectable }
];

function useService(service) {
    const state$ = new BehaviorSubject(service.state);
    const sub = service.subscribe((s) => state$.next(s));
    state$.pipe(filter(({ changed }) => changed), shareReplay(1), finalize(() => sub.unsubscribe()));
    return { state$, send: service.send, service };
}

/**
 * Generated bundle index. Do not edit.
 */

export { XstateAngular, useService };
//# sourceMappingURL=xstate-angular.js.map
